package com.mints.goodmoney.ui.activitys

import android.os.Bundle
import android.text.TextUtils
import android.view.View
import cn.jzvd.Jzvd
import com.dingmouren.videowallpaper.VideoWallpaper
import com.downloader.Error
import com.downloader.OnDownloadListener
import com.downloader.PRDownloader
import com.mints.goodmoney.R
import com.mints.goodmoney.ad.video.VideoAdingManager
import com.mints.goodmoney.common.Constant
import com.mints.goodmoney.manager.UserManager
import com.mints.goodmoney.mvp.model.VideoAdingBean
import com.mints.goodmoney.mvp.presenters.KylVideoPresenter
import com.mints.goodmoney.mvp.views.KylVideoView
import com.mints.goodmoney.ui.activitys.base.BaseActivity
import com.mints.goodmoney.utils.CacheUtil
import com.mints.goodmoney.utils.LogUtil
import com.mints.goodmoney.utils.MD5
import kotlinx.android.synthetic.main.activity_kyl_video.*

/**
 * 描述：酷音乐视频
 * 作者：孟崔广
 * 时间：2021/1/4 15:09
 */
class KylVideoActivity : BaseActivity(), View.OnClickListener, KylVideoView {

    companion object {
        const val VIDEO_URL = "video_url"
    }

    private lateinit var videoUrl: String

    private val kylVideoPresenter by lazy { KylVideoPresenter() }
    private lateinit var videoAdingManager: VideoAdingManager
    private var loadVideoFailCount = 0

    private val videoWallpaper by lazy { VideoWallpaper() }
    private var downloadId: Int = -1//video下载id
    private var downloadVideoSucFlag = false// 下载视频成功
    private var cachePath = ""// cache路径
    private var videoName = ""// 视频名称


    override fun getContentViewLayoutID() = R.layout.activity_kyl_video

    override fun isApplyKitKatTranslucency() = false

    override fun getBundleExtras(extras: Bundle?) {
        super.getBundleExtras(extras)
        extras?.let {
            videoUrl = it.getString(VIDEO_URL, "")
        }
    }

    override fun initViewsAndEvents() {
        if (TextUtils.isEmpty(videoUrl)) {
            showToast("加载失败，请稍后重试.")
            finish()
            return
        }
        cachePath = cacheDir.path + "/"
        videoAdingManager = VideoAdingManager.getInstance(this)
        initListener()
        loadVideo(videoUrl)
    }

    private fun loadVideo(videoUrl: String) {
        jzvdStd?.setUp(videoUrl, "")
        jzvdStd?.startVideo()
//        jzvdStd.setVideoRestart(false)
//        jzvdStd.setJzVideoStatusListener(object : JZVideoPlayerStandardLoopVideo.JZVideoStatusListener {
//           override fun jzVideoPlaying() {
//            }
//        })
    }

    override fun onBackPressed() {
        if (Jzvd.backPress()) {
            return
        }
        super.onBackPressed()
    }

    override fun onResume() {
        super.onResume()

        if (!isFinishing()) {
            Jzvd.goOnPlayOnResume()
        }
    }

    override fun onPause() {
        super.onPause()

        Jzvd.goOnPlayOnPause()
    }

    override fun onDestroy() {
        super.onDestroy()
        Jzvd.releaseAllVideos()
        jzvdStd?.removeAllViews()

        if (downloadId != -1) {
            PRDownloader.cancel(downloadId)
        }

        //删除缓存文件
        CacheUtil.clearAllCache(context)

        kylVideoPresenter.detachView()
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.ivBack -> finish()
            R.id.ivSetting -> {
                awardVideo()
                downloadVideoToCacheFile()
            }
        }
    }

    private fun downloadVideoToCacheFile() {
        videoName = MD5.GetMD5Code(System.currentTimeMillis().toString() + UserManager.getInstance().userID) + ".mp4"
        downloadId = PRDownloader.download(videoUrl, cachePath, videoName)
                .build()
//                .setOnStartOrResumeListener { }
//                .setOnPauseListener {
//                }
//                .setOnCancelListener {
//                }
//                .setOnProgressListener { progress ->
//                }
                .start(object : OnDownloadListener {
                    override fun onDownloadComplete() {
                        downloadVideoSucFlag = true
                    }

                    override fun onError(error: Error) {

                    }
                })
    }

    private fun initListener() {
        ivBack.setOnClickListener(this)
        ivSetting.setOnClickListener(this)
    }

    /**
     * 获取激励视频
     */
    private fun awardVideo() {
        if (videoAdingManager.videoFinishFlag) {
            showToast("今日视频已看完，请明天再来吧")
            return
        }
        loadVideoFailCount = 0
        showLoading("加载中...")

        val bean = VideoAdingBean()
        bean.carrierType = Constant.CARRIER_KUYINYUE_VEDIO
        loadVideo(videoAdingManager, bean, true)
    }

    /**
     * 加载激励视频
     */
    private fun loadVideo(videoAdingManager: VideoAdingManager, bean: VideoAdingBean, isFirstLoad: Boolean) {
        videoAdingManager.setVideoAdingListener(object : VideoAdingManager.VideoAdingListener {
            override fun videoAdingListenerError(adType: String) {
                showToast("加载超时，请休息一下")
                hideLoading()
            }

            override fun videoAdingListenerFail(adType: String) {
                loadVideoFailCount++
                if (loadVideoFailCount >= 2) {
                    hideLoading()
                    showToast("加载失败，请稍后重试!")
                } else {
                    loadVideo(videoAdingManager, bean, false)
                }
            }

            override fun videoAdingListenerSuccess(adType: String) {
                hideLoading()
                videoAdingSuccess(adType)
            }
        })
        if (isFirstLoad) {
            // 首页按权重加载
            videoAdingManager.loadAding(this, bean)
        } else {
            // 失败按优先级加载
            videoAdingManager.loadFailAding(this, bean)
        }
    }

    private fun videoAdingSuccess(adType: String) {
        if (downloadVideoSucFlag) {
            LogUtil.d("KylVideoActivity", "name:-->" + cachePath + videoName)
            try {
                showToast("设置壁纸成功")
                videoWallpaper.setToWallPaper(this, cachePath + videoName)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }
}
